@@ -1,12 +1,11 @@
 #
-#   Pyrex - Tables describing slots in the type object
-#           and associated know-how.
+#   Tables describing slots in the CPython type object
+#   and associated know-how.
 #
 
 import Naming
 import PyrexTypes
 import StringEncoding
-import sys
 
 invisible = ['__cinit__', '__dealloc__', '__richcmp__',
              '__nonzero__', '__bool__']
@@ -32,6 +31,8 @@ class Signature(object):
     #    'b'  bint
     #    'I'  int *
     #    'l'  long
+    #    'f'  float
+    #    'd'  double
     #    'h'  Py_hash_t
     #    'z'  Py_ssize_t
     #    'Z'  Py_ssize_t *
@@ -53,6 +54,8 @@ class Signature(object):
         'b': PyrexTypes.c_bint_type,
         'I': PyrexTypes.c_int_ptr_type,
         'l': PyrexTypes.c_long_type,
+        'f': PyrexTypes.c_float_type,
+        'd': PyrexTypes.c_double_type,
         'h': PyrexTypes.c_py_hash_t_type,
         'z': PyrexTypes.c_py_ssize_t_type,
         'Z': PyrexTypes.c_py_ssize_t_ptr_type,
@@ -64,6 +67,9 @@ class Signature(object):
         # and are not looked up in here
     }
 
+    type_to_format_map = dict([(type_, format_)
+                                   for format_, type_ in format_map.iteritems()])
+
     error_value_map = {
         'O': "NULL",
         'T': "NULL",
@@ -87,6 +93,7 @@ class Signature(object):
         self.fixed_arg_format = arg_format
         self.ret_format = ret_format
         self.error_value = self.error_value_map.get(ret_format, None)
+        self.is_staticmethod = False
 
     def num_fixed_args(self):
         return len(self.fixed_arg_format)
@@ -105,6 +112,11 @@ class Signature(object):
     def return_type(self):
         return self.format_map[self.ret_format]
 
+    def format_from_type(self, arg_type):
+        if arg_type.is_pyobject:
+            arg_type = PyrexTypes.py_object_type
+        return self.type_to_format_map[arg_type]
+
     def exception_value(self):
         return self.error_value_map.get(self.ret_format)
 
@@ -137,6 +149,9 @@ class Signature(object):
                     return [method_noargs]
             elif full_args in ["OO", "TO"] and not self.has_generic_args:
                 return [method_onearg]
+
+            if self.is_staticmethod:
+                return [method_varargs, method_keywords]
         return None
 
 
@@ -145,14 +160,16 @@ class SlotDescriptor(object):
     #
     #  slot_name    string           Member name of the slot in the type object
     #  is_initialised_dynamically    Is initialised by code in the module init function
+    #  is_inherited                  Is inherited by subtypes (see PyType_Ready())
     #  py3                           Indicates presence of slot in Python 3
     #  py2                           Indicates presence of slot in Python 2
     #  ifdef                         Full #ifdef string that slot is wrapped in. Using this causes py3, py2 and flags to be ignored.)
 
-    def __init__(self, slot_name, dynamic=0,
+    def __init__(self, slot_name, dynamic=False, inherited=False,
                  py3=True, py2=True, ifdef=None):
         self.slot_name = slot_name
         self.is_initialised_dynamically = dynamic
+        self.is_inherited = inherited
         self.ifdef = ifdef
         self.py3 = py3
         self.py2 = py2
@@ -171,10 +188,29 @@ class SlotDescriptor(object):
         return guard
 
     def generate(self, scope, code):
+        end_pypy_guard = False
         if self.is_initialised_dynamically:
-            value = 0
+            value = "0"
         else:
             value = self.slot_code(scope)
+            if value == "0" and self.is_inherited:
+                # PyPy currently has a broken PyType_Ready() that fails to
+                # inherit some slots.  To work around this, we explicitly
+                # set inherited slots here, but only in PyPy since CPython
+                # handles this better than we do.
+                inherited_value = value
+                current_scope = scope
+                while (inherited_value == "0"
+                       and current_scope.parent_type
+                       and current_scope.parent_type.base_type
+                       and current_scope.parent_type.base_type.scope):
+                    current_scope = current_scope.parent_type.base_type.scope
+                    inherited_value = self.slot_code(current_scope)
+                if inherited_value != "0":
+                    code.putln("#if CYTHON_COMPILING_IN_PYPY")
+                    code.putln("%s, /*%s*/" % (inherited_value, self.slot_name))
+                    code.putln("#else")
+                    end_pypy_guard = True
         preprocessor_guard = self.preprocessor_guard_code()
         if preprocessor_guard:
             code.putln(preprocessor_guard)
@@ -184,6 +220,8 @@ class SlotDescriptor(object):
             code.putln("0, /*reserved*/")
         if preprocessor_guard:
             code.putln("#endif")
+        if end_pypy_guard:
+            code.putln("#endif")
 
     # Some C implementations have trouble statically
     # initialising a global with a pointer to an extern
@@ -199,7 +237,7 @@ class SlotDescriptor(object):
                     self.slot_name,
                     value
                     )
-            )
+                )
 
 
 class FixedSlot(SlotDescriptor):
@@ -230,8 +268,9 @@ class MethodSlot(SlotDescriptor):
     #  alternatives [string]         Alternative list of __xxx__ names for the method
 
     def __init__(self, signature, slot_name, method_name, fallback=None,
-                 py3=True, py2=True, ifdef=None):
-        SlotDescriptor.__init__(self, slot_name, py3=py3, py2=py2, ifdef=ifdef)
+                 py3=True, py2=True, ifdef=None, inherited=True):
+        SlotDescriptor.__init__(self, slot_name, py3=py3, py2=py2,
+                                ifdef=ifdef, inherited=inherited)
         self.signature = signature
         self.slot_name = slot_name
         self.method_name = method_name
@@ -300,7 +339,8 @@ class ConstructorSlot(InternalMethodSlot):
         self.method = method
 
     def slot_code(self, scope):
-        if scope.parent_type.base_type \
+        if self.slot_name != 'tp_new' \
+            and scope.parent_type.base_type \
             and not scope.has_pyobject_attrs \
             and not scope.lookup_here(self.method):
             # if the type does not have object attributes, it can
@@ -338,8 +378,15 @@ class TypeFlagsSlot(SlotDescriptor):
     #  Descriptor for the type flags slot.
 
     def slot_code(self, scope):
-        value = "Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER"
-        if not scope.directives['final']:
+        value = "Py_TPFLAGS_DEFAULT"
+        if scope.directives['type_version_tag']:
+            # it's not in 'Py_TPFLAGS_DEFAULT' in Py2
+            value += "|Py_TPFLAGS_HAVE_VERSION_TAG"
+        else:
+            # it's enabled in 'Py_TPFLAGS_DEFAULT' in Py3
+            value = "(%s&~Py_TPFLAGS_HAVE_VERSION_TAG)" % value
+        value += "|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER"
+        if not scope.parent_type.is_final_type:
             value += "|Py_TPFLAGS_BASETYPE"
         if scope.needs_gc():
             value += "|Py_TPFLAGS_HAVE_GC"
@@ -371,21 +418,30 @@ class SuiteSlot(SlotDescriptor):
         self.slot_type = slot_type
         substructures.append(self)
 
+    def is_empty(self, scope):
+        for slot in self.sub_slots:
+            if slot.slot_code(scope) != "0":
+                return False
+        return True
+
     def substructure_cname(self, scope):
         return "%s%s_%s" % (Naming.pyrex_prefix, self.slot_name, scope.class_name)
 
     def slot_code(self, scope):
-        return "&%s" % self.substructure_cname(scope)
+        if not self.is_empty(scope):
+            return "&%s" % self.substructure_cname(scope)
+        return "0"
 
     def generate_substructure(self, scope, code):
-        code.putln("")
-        code.putln(
-            "static %s %s = {" % (
-                self.slot_type,
-                self.substructure_cname(scope)))
-        for slot in self.sub_slots:
-            slot.generate(scope, code)
-        code.putln("};")
+        if not self.is_empty(scope):
+            code.putln("")
+            code.putln(
+                "static %s %s = {" % (
+                    self.slot_type,
+                    self.substructure_cname(scope)))
+            for slot in self.sub_slots:
+                slot.generate(scope, code)
+            code.putln("};")
 
 substructures = []   # List of all SuiteSlot instances
 
@@ -454,11 +510,13 @@ def get_special_method_signature(name):
     else:
         return None
 
+
 def get_property_accessor_signature(name):
     #  Return signature of accessor for an extension type
     #  property, else None.
     return property_accessor_signatures.get(name)
 
+
 def get_base_slot_function(scope, slot):
     #  Returns the function implementing this slot in the baseclass.
     #  This is useful for enabling the compiler to optimize calls
@@ -472,6 +530,18 @@ def get_base_slot_function(scope, slot):
                 return parent_slot
     return None
 
+
+def get_slot_function(scope, slot):
+    #  Returns the function implementing this slot in the baseclass.
+    #  This is useful for enabling the compiler to optimize calls
+    #  that recursively climb the class hierarchy.
+    slot_code = slot.slot_code(scope)
+    if slot_code != '0':
+        entry = scope.parent_scope.lookup_here(scope.parent_type.name)
+        if entry.visibility != 'extern':
+            return slot_code
+    return None
+
 #------------------------------------------------------------------------------------------
 #
 #  Signatures for generic Python functions and methods.
@@ -670,7 +740,7 @@ slot_table = (
     SuiteSlot(PySequenceMethods, "PySequenceMethods", "tp_as_sequence"),
     SuiteSlot(PyMappingMethods, "PyMappingMethods", "tp_as_mapping"),
 
-    MethodSlot(hashfunc, "tp_hash", "__hash__"),
+    MethodSlot(hashfunc, "tp_hash", "__hash__", inherited=False),    # Py3 checks for __richcmp__
     MethodSlot(callfunc, "tp_call", "__call__"),
     MethodSlot(reprfunc, "tp_str", "__str__"),
 
@@ -686,7 +756,7 @@ slot_table = (
     GCDependentSlot("tp_clear"),
 
     # Later -- synthesize a method to split into separate ops?
-    MethodSlot(richcmpfunc, "tp_richcompare", "__richcmp__"),
+    MethodSlot(richcmpfunc, "tp_richcompare", "__richcmp__", inherited=False),  # Py3 checks for __hash__
 
     EmptySlot("tp_weaklistoffset"),
 
@@ -718,6 +788,7 @@ slot_table = (
     EmptySlot("tp_weaklist"),
     EmptySlot("tp_del"),
     EmptySlot("tp_version_tag", ifdef="PY_VERSION_HEX >= 0x02060000"),
+    EmptySlot("tp_finalize", ifdef="PY_VERSION_HEX >= 0x030400a1 && defined(Py_TPFLAGS_HAVE_FINALIZE)"),
 )
 
 #------------------------------------------------------------------------------------------
